#include <string>
#include <vector>
#include <iostream>

using namespace std;

/**
 A classe CGIpair deriva da classe pair da STL
 */
class CGIpair : public pair<string, string> {
public:
  CGIpair() {}
  CGIpair(string name, string value) {
    first = decodeURLString(name);
    second = decodeURLString(value);
  }
  /**
    Conversão automática de tipo para usar em testes booleanos
  */
  operator bool() const {
    return (first.length() != 0);
  } 
private:
  /**
    Decodifica caracteres especiais de uma string vinda da URL
    O caractere + é substituido pelo espaço
    Quando Encontra o caractere % os 2 caracteres que o seguem
    são a representação hexadecimal do caractere original.
    Ex.:    %3F   representa o caractere  '?' (interrogação)
           %25   representa o caractere  '%' (porcentagem)
  */
  static string decodeURLString(string URLstr) {
    const int len = URLstr.length();
    string result;
    for(int i = 0; i < len; i++) {
      if(URLstr[i] == '+') {
        result += ' ';
      } else if(URLstr[i] == '%') {
        result +=
             translateHex(URLstr[i + 1]) * 16 +   //Aki múltiplica por 16 o número decimal retornado
             translateHex(URLstr[i + 2]);
        i += 2; // Move o i para uma posição logo em seguida dos caracteres hexa
      } else { // Um caractere comum
        result += URLstr[i];
      }  
    }
    return result;
  }
  // Converte um caractere hexadecimal para sua representação decimal
  // Usado por decodeURLString():
  static char translateHex(char hex) {
    if(hex >= 'A') {
      return (hex & 0xdf) - 'A' + 10;
    } else {
      return hex - '0';
    }  
  }
};
/**
 Separa uma query CGI e coloca ela dentro de um and turns it into an
 vector de CGIpairs. O vector, assim como um map, tem o operador [] para buscas
 O vector é usado ao invés do map porque ele mantém a ordem original
 dos campos dispostos no formulário.
*/
class CGImap : public vector<CGIpair> {
  //variáveis auxiliares
  int index;
  string gq; 
  /**Previne atribuição*/
  void operator=(CGImap&);
  /**Previne construção por cópia*/
  CGImap(CGImap&);
public:
  /**Construtor da classe CGImap*/
  CGImap(string query): index(0), gq(query){
    //Vai separando cada par de campo/valor e colocando no vector
    CGIpair p;
    while((p = nextPair()) != 0) {
      push_back(p); 
    }  
  }
  /**
    Aqui se sobrecarrega o operador [] para realizar a busca de chaves
    e retornar seu valor
  */  
  string operator[](const string& key) {
    iterator i = begin();
    while(i != end()) {
      if((*i).first == key) {
        return (*i).second;
      }  
      i++;
    }
    return string(); // Caso não ache a chave retorna uma string em branco
  }
  /**
    Uma função para mostrar o conteúdo do CGImap
    em uma stream. Se pode passar uma string para quebra de linha
    e por default é selecionada a quebra de linha html <br>
  */
  void dump(ostream& o, string nl = "<br>") {
    for(iterator i = begin(); i != end(); i++) {
      o << (*i).first << " = " << (*i).second << nl;
    }
  }
private:
  /**
    Produces name-value pairs from the query
    string. Returns an empty Pair when there's
    no more query string left:
  */  
  CGIpair nextPair() {
    if(gq.length() == 0) {
      return CGIpair(); // Error, return empty
    }  
    if(gq.find('=') == string::npos) {
      return CGIpair(); // Error, return empty
    }  
    string name = gq.substr(0, gq.find('='));
    gq = gq.substr(gq.find('=') + 1);
    string value = gq.substr(0, gq.find('&'));
    gq = gq.substr(gq.find('&') + 1);
    return CGIpair(name, value);
  }
};

/**
  Classe auxiliar para pegar os dados do POST
*/   
class Post : public string {
public: 
  Post() {
    // Para um CGI do tipo POST o servidor guarda
    // o tamanho da string com o conteúdo
    // em uma variável de ambiente chamada CONTENT_LENGTH:
    char* clen = getenv("CONTENT_LENGTH"); //após essa chamada clen aponta para uma string "C-Like" ou fica NULA
    if(clen == NULL ) { 
      throw(string("CONTENT_LENGTH=Zero. Tenha certeza que isso é um POST e não um GET")) ;
    }
    int len = atoi(clen); //Converte a string "C-Like" para inteiro
    char* s = new char[len]; //Cria uma string "C-Like" para ler os dados do POST
    // Coloca os dados em s.
    // Note a entrada padrão, que normalmente é o teclado, em um CGI representa os dados
    // Enviados pelo formulário submetido pelo usuário
    cin.read(s, len); 
    append(s, len); // Adiciona os dados de s para a string desse objeto
    delete []s; 
  }
}; 
